require 'spec_helper'

describe 'openldap::server' do
  on_supported_os.each do |os, os_facts|
    context "on #{os}" do
      let(:facts) { os_facts }

      it { is_expected.to compile.with_all_deps }

      context 'with provider => olc' do
        let(:params) { { 'provider' => 'olc' } }

        it { is_expected.to contain_class('openldap::utils') }
      end

      context 'with provider => augeas' do
        let(:params) { { 'provider' => 'augeas' } }

        it { is_expected.not_to contain_class('openldap::utils') }
      end

      context 'with provider => slapdconf' do
        let(:params) { { 'provider' => 'slapdconf' } }

        it { is_expected.to compile.and_raise_error(%r{provider}) }
      end

      case os_facts[:osfamily]
      when 'Debian'
        it do
          is_expected.to contain_package('slapd').with_ensure('installed')
          is_expected.to contain_file('/var/cache/debconf/slapd.preseed').with(
            'ensure' => 'file',
            'owner'  => 'root',
            'group'  => 'root',
            'mode'   => '0644',
          )
        end
      end
    end
  end
end
